#!/usr/bin/perl

$version="1.0";

# Changelog
#
# 2004-01-16 - Rudolf Potucek 
#
#   Added more detailed comments before posting the script for general use.
#
# 2004-12-14 - Rudolf Potucek 
#
#   Created the script in its original form.
#
use Getopt::Long;

GetOptions qw( -h --help -V --version);

if ($opt_V || $opt_version) {
    print "$0 Version $version\n";
    exit;
}

if ($opt_h || $opt_help || ! scalar(@ARGV)) {
    print <<EOT;
# USAGE $0 input [input ...]
#
# Reorder the numbers of atoms in a topology file. This script is motivated
# by the occasional need to change topology to accomodate different charge
# groups. Note that this script will NOT alter the charges/charge groups,
# that process remains manual.
#
# To use this script, change the order of the lines in the [atoms] section
# of a topology file to be in the desired order but DO NOT alter the atom
# numbers. The script assumes that the atoms in the [atoms] section should
# be ordered 1,2,3, ... n. It will take the original atom numbers it finds 
# in column 1 and renumber atom references in the [bonds], [pairs], [angles] 
# and [dihedral] blocks accordingly. Output is written to stdout.
#
# NOTE: due to this approach the [atoms] section needs to precede all other 
# sections!
#
EOT
    exit;
}

# Loop over all input files. Note that while output goes to stdout all
# input files would be merged on stdout! I.e., generally you would want
# to run this on a single file.
#
foreach $file (@ARGV) {
    
    # Clear the atomnr array just in case there are multiple input files!
    @atomnr=();
    
    open(FH,"<$file")
      or die "Could not open $file!\n";
    
    while ($line=<FH>) {
	# Comments ... just print them
	if ($line =~ /^\s*[;#]/) {
	    print $line;
	    next;
	}
	
	# [atoms] block ... here we read the desired order ...
	# to do so, the old atom number found in column 1 is stored in
	# the atomnr array with the OLD number as index and the NEW number
	# as value. 
	# 
	if ($line =~ /^\s*\[\s*atoms\s*\]/) {
	    print stderr "### Entering atoms block\n";
	    print $line;
	    
	    $atoms=1; $count=1;
	    while (1) {
		$line=<FH>;
		
		# Comments ... just print them
		if ($line =~ /^\s*[;#]/) {
		    print $line;
		    next;
		}
		
		# Empty line ... end of block
		if ($line =~ /^\s*$/) {
		    print stderr "### Exiting atoms block\n";
		    print $line;
		    last;
		}
		
		# All other lines should be atoms ... we are really only
		# interested in the first entry
		$line=~s/^\s*//;
		$line=~s/\s*$//;
		@line=split /\s+/,$line;
		
		
		$atomnr[$line[0]]=$count; 
		$line[0]=$count;
		$count++;
		print join("\t",@line),"\n";
	    }
	}	
	
	# [bonds] block ... first two columns renumbered
	if ($line =~ /^\s*\[\s*bonds\s*\]/) {
	    print stderr "### Entering bonds block\n";
	    print $line;

	    renum(2,FH);
	    print stderr "### Leaving bonds block\n";
	}
	
	# [pairs] block ... first two columns renumbered
	if ($line =~ /^\s*\[\s*pairs\s*\]/) {
	    print stderr "### Entering pairs block\n";
	    print $line;

	    renum(2,FH);
	    print stderr "### Leaving pairs block\n";
	}
	
	# [angles] block ... first three columns renumbered
	if ($line =~ /^\s*\[\s*angles\s*\]/) {
	    print stderr "### Entering angles block\n";
	    print $line;

	    renum(3,FH);
	    print stderr "### Leaving angles block\n";
	}
	
	# [dihedrals] block ... first two columns renumbered
	if ($line =~ /^\s*\[\s*dihedrals\s*\]/) {
	    print stderr "### Entering dihedrals block\n";
	    print $line;

	    renum(4,FH);
	    print stderr "### Leaving dihedrals block\n";
	}
	
    }
    close(FH);
}
    
# Read a block and renumber the first 'cols' columns with the 
# numbers from the 'atomnr' array ... note that the filehandle
# to read from and the number of columns to replace are passed
# as parameters but the atomnr array is a global variable!
#
# The actual replacing is done by simply indexing the atomnr array
# with the OLD index found and thus retrieving the new index desired.
#
sub renum {
    my($cols,$fh)=@_;
    my($ii);
    
    while (1) {
	$line=<FH>;
	
	# Comments ... just print them
	if ($line =~ /^\s*[;#]/) {
	    print $line;
	    next;
	}
	
	# Empty line ... end of block
	if ($line =~ /^\s*$/) {
	    print $line;
	    last;
	}
	
	# Here we do the replacing ...
	$line=~s/^\s*//;
	$line=~s/\s*$//;
	@line=split /\s+/,$line;
	
	for ($ii=0; $ii<$cols; $ii++) {
	    $line[$ii]=$atomnr[$line[$ii]];
	}	    
	print join("\t",@line),"\n";
    }
}
